//+------------------------------------------------------------------+
//|                            Volume with Custom Moving Average.mq4 |
//|                      Copyright  2004, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//|                                              2nd MA added by cja |
//+------------------------------------------------------------------+
#property copyright "Copyright  2004, MetaQuotes Software Corp. Modified RonT"
#property link      "http://www.metaquotes.net/"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1 Red
#property indicator_color2 Green
#property indicator_color3 White
#property indicator_color4 Gold

#property indicator_width1 2
#property indicator_width2 2
#property indicator_width3 2
#property indicator_width4 2


// User input
extern int MA_Period=15;
extern int MA_Shift=0;
extern int MA_Method=1;

extern int MA_Period2=50;
extern int MA_Shift2=0;
extern int MA_Method2=1;

extern bool Show_Label_Digits = false;

// Buffers
double VolBuffer1[];  // value down
double VolBuffer2[];  // value up
double VolBuffer3[];  // moving average
double VolBuffer4[];  // moving average

//----
int ExtCountedBars=0;
int lastcolor=0;


//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   int    draw_begin;
   string short_name;
   string short_name2;

   // indicator buffers mapping, drawing settings and Shift
   
   // Histogram downArrow Red
   SetIndexBuffer(0,VolBuffer1);
   SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexShift(0,MA_Shift);

   // Histogram upArrow Green
   SetIndexBuffer(1,VolBuffer2); 
   SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexShift(1,MA_Shift);

   // Moving average line Black
   SetIndexBuffer(2,VolBuffer3);
   SetIndexStyle(2,DRAW_LINE);
   SetIndexShift(2,MA_Shift);

   // Moving average line Black
   SetIndexBuffer(3,VolBuffer4);
   SetIndexStyle(3,DRAW_LINE);
   SetIndexShift(3,MA_Shift2);

   IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS));
   
   if(Show_Label_Digits==false){
   SetIndexLabel(0,"");
   SetIndexLabel(1,"");
   SetIndexLabel(2,"");
   SetIndexLabel(3,"");
   }
   
   if(Show_Label_Digits==true){
   SetIndexLabel(0,"VolBuffer1");
   SetIndexLabel(1,"VolBuffer2");
   SetIndexLabel(2,"VolBuffer3");
   SetIndexLabel(3,"VolBuffer4");
   }


   if(MA_Period<2) MA_Period=2;
   draw_begin=MA_Period-1;

   switch(MA_Method)
     {
      case 1  : short_name="EMA(" ;  draw_begin=0; break;
      case 2  : short_name="SMMA(";                break;
      case 3  : short_name="LWMA(";                break;
      default : short_name="SMA(" ;  MA_Method=0;
     }
     
    if(MA_Period2<2) MA_Period2=2;
   draw_begin=MA_Period2-1;

   switch(MA_Method2)
     {
      case 1  : short_name2="EMA(" ;  draw_begin=0; break;
      case 2  : short_name2="SMMA(";                break;
      case 3  : short_name2="LWMA(";                break;
      default : short_name2="SMA(" ;  MA_Method2=0;
     }  
   IndicatorShortName(short_name+MA_Period+" )( "+short_name2+MA_Period2+")");

   SetIndexDrawBegin(0,draw_begin);


   return(0);
  }


//+------------------------------------------------------------------+
//| Main                                                             |
//+------------------------------------------------------------------+
int start()
  {
  
   if(Bars<=MA_Period || Bars<=MA_Period2 ) return(0);
   ExtCountedBars=IndicatorCounted();

// check for possible errors
   if (ExtCountedBars<0) return(-1);

// last counted bar will be recounted
   if (ExtCountedBars>0) ExtCountedBars--;

   switch(MA_Method)
     {
      case 0 : sma();  break;
      case 1 : ema();  break;
      case 2 : smma(); break;
      case 3 : lwma();
     }
   
   switch(MA_Method2)
     {
      case 0 : sma();  break;
      case 1 : ema();  break;
      case 2 : smma(); break;
      case 3 : lwma();
     }  
   return(0);
  }

//+------------------------------------------------------------------+
//| Simple Moving Average                                            |
//+------------------------------------------------------------------+
void sma()
  {
   double sum=0;
   double sum2=0;
   int    i,pos=Bars-ExtCountedBars-1,pos2=Bars-ExtCountedBars-1;

   // initial accumulation
   if(pos<MA_Period) pos=MA_Period;
   for(i=1;i<MA_Period;i++,pos--)
      sum+=Volume[pos];
   
   while(pos>=0)
     {
      sum+=Volume[pos];
      VolBuffer3[pos]=sum/MA_Period;
      sum-=Volume[pos+MA_Period-1];
      Vcolor(pos);
    pos--;
    
     // initial accumulation
   if(pos2<MA_Period2) pos2=MA_Period2;
   for(i=1;i<MA_Period2;i++,pos2--)
      sum2+=Volume[pos2];
   }
   while(pos2>=0)
     {
      sum2+=Volume[pos2];
      VolBuffer4[pos2]=sum2/MA_Period2;
      sum2-=Volume[pos2+MA_Period2-1];
      Vcolor(pos2);
    pos2--;
     }

   // zero initial bars
   if(ExtCountedBars<1)
      for(i=1;i<MA_Period;i++) VolBuffer1[Bars-i]=0;
      for(i=1;i<MA_Period2;i++) VolBuffer1[Bars-i]=0;
  }



//+------------------------------------------------------------------+
//| Exponential Moving Average                                       |
//+------------------------------------------------------------------+
void ema()
  {
   double pr=2.0/(MA_Period+1);
   double pr2=2.0/(MA_Period2+1);
   int    pos=Bars-2;
   int    pos2=Bars-2;

   if(ExtCountedBars>2) pos=Bars-ExtCountedBars-1;

   while(pos>=0)
     {
      if(pos==Bars-2) VolBuffer3[pos+1]=Volume[pos+1];
      VolBuffer3[pos]=Volume[pos]*pr+VolBuffer3[pos+1]*(1-pr);
      Vcolor(pos);
      pos--;
      }
   if(ExtCountedBars>2) pos2=Bars-ExtCountedBars-1;   
     while(pos2>=0)
     {  
       if(pos2==Bars-2) VolBuffer4[pos2+1]=Volume[pos2+1];
      VolBuffer4[pos2]=Volume[pos2]*pr2+VolBuffer4[pos2+1]*(1-pr2);
      Vcolor(pos2);
    pos2--;
     }
  }



//+------------------------------------------------------------------+
//| Smoothed Moving Average                                          |
//+------------------------------------------------------------------+
void smma()
  {
   double sum=0;
   double sum2=0;
   int    i,k,pos=Bars-ExtCountedBars+1,pos2=Bars-ExtCountedBars+1;

   pos=Bars-MA_Period;
   if(pos>Bars-ExtCountedBars) pos=Bars-ExtCountedBars;
   while(pos>=0)
     {
      if(pos==Bars-MA_Period)
        {
         // initial accumulation
         for(i=0,k=pos;i<MA_Period;i++,k++)
           {
            sum+=Volume[k];
            // zero initial bars
            VolBuffer3[k]=0;
           }
        }
      else sum=VolBuffer3[pos+1]*(MA_Period-1)+Volume[pos];
      VolBuffer3[pos]=sum/MA_Period;
     pos--; 
     } 
   pos2=Bars-MA_Period2;
   if(pos2>Bars-ExtCountedBars) pos2=Bars-ExtCountedBars;
   while(pos2>=0)
     {
      if(pos2==Bars-MA_Period2)
        {
         // initial accumulation
         for(i=0,k=pos2;i<MA_Period2;i++,k++)
           {
            sum2+=Volume[k];
            // zero initial bars
            VolBuffer4[k]=0;
           }
        }
      else sum2=VolBuffer4[pos2+1]*(MA_Period2-1)+Volume[pos2];
      VolBuffer4[pos2]=sum2/MA_Period2;
    pos2--;
     }
  }



//+------------------------------------------------------------------+
//| Linear Weighted Moving Average                                   |
//+------------------------------------------------------------------+
void lwma()
  {
   double sum=0.0,lsum=0.0;
   double price;
   double sum2=0.0,lsum2=0.0;
   double price2;
   
   int    i,weight=0,weight2=0,pos=Bars-ExtCountedBars-1,pos2=Bars-ExtCountedBars-1;
//---- initial accumulation
   if(pos<MA_Period) pos=MA_Period;
   for(i=1;i<=MA_Period;i++,pos--)
     {
      price=Volume[pos];
      sum+=price*i;
      lsum+=price;
      weight+=i;
     }
//---- main calculation loop
   pos++;
   i=pos+MA_Period;
   while(pos>=0)
     {
      VolBuffer3[pos]=sum/weight;
      if(pos==0) break;
      pos--;
      i--;
      price=Volume[pos];
      sum=sum-lsum+price*MA_Period;
      lsum-=Volume[i];
      lsum+=price;
     }
//---- zero initial bars
   if(ExtCountedBars<1)
      for(i=1;i<MA_Period;i++) VolBuffer3[Bars-i]=0;
    

if(pos2<MA_Period2) pos2=MA_Period2;
   for(i=1;i<=MA_Period2;i++,pos2--)
     {
      price2=Volume[pos2];
      sum2+=price2*i;
      lsum2+=price2;
      weight2+=i;
     }
//---- main calculation loop
   pos2++;
   i=pos2+MA_Period2;
   while(pos2>=0)
     {
      VolBuffer4[pos2]=sum2/weight2;
      if(pos2==0) break;
      pos2--;
      i--;
      price2=Volume[pos2];
      sum2=sum2-lsum2+price2*MA_Period2;
      lsum2-=Volume[i];
      lsum2+=price2;
     }
//---- zero initial bars
   if(ExtCountedBars<1)
      for(i=1;i<MA_Period2;i++) VolBuffer4[Bars-i]=0;
  }

//+------------------------------------------------------------------+
//| Color depends on gain or loss and previous volume                |
//+------------------------------------------------------------------+
// 1 - histo down red
// 2 - histo up green
// 3 - line white

void Vcolor(int p)
  {
 
   if (Volume[p+1]>Volume[p])
     {
      VolBuffer1[p]=Volume[p];
      VolBuffer2[p]=0;
      lastcolor=Red;
     }

   if (Volume[p+1]<Volume[p])
     {
      VolBuffer1[p]=0;
      VolBuffer2[p]=Volume[p];
      lastcolor=Green;
     }        

   if (Volume[p+1]==Volume[p])
     {
      if ( lastcolor==Red )
        {
         VolBuffer1[p]=Volume[p];
         VolBuffer2[p]=0;
        }
      if ( lastcolor==Green )
        {
         VolBuffer1[p]=0;
         VolBuffer2[p]=Volume[p];
        }
     }
  }

//+------------------------------------------------------------------+